@@ -25,7 +25,6 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #include <errno.h>
 
 #include "dz80.h"
-#include "symtab.h"
 #include "blocks.h"
 #include "config.h"
 #include "utils.h"
@@ -296,14 +295,14 @@ else if((a>=0xb0)&&(a<0xc0)&&(edb0[a-0xb0][0]!='0'))FP(fx,"%s",edb0[a-0xb0]);
 	}
 
 	if(stro[0] == 0) {
-		FP(fx,rstr);
+		FP(fx,"%s",rstr);
 		return(2);
 	} else {
 		r=ckrange(2,bstr);
 		if(r==-1) {
 			return defb(-1);
 		} else {
-			FP(fx,rstr);
+			FP(fx,"%s",rstr);
 			FP(fx,stro,bstr);
 		}
 		return(4);
@@ -973,7 +972,7 @@ int main(int argc, char *argv[])
 
 	if(f2) {
 		msg(2, "Writing symbol file\n");
-		symbol_export(f2);
+		symbol_export(f2, a_symbol_order);
 		fclose(f2);
 	}
 	/***************************pass 3*********************************/
