@@ -1,5 +1,7 @@
 #include <stdio.h>
 
+#include "symtab.h"
+
 #define MAX_LINE_LEN	1024
 
 struct comm{
@@ -22,6 +24,7 @@ extern char *a_output;
 
 extern char *a_symoutput;
 extern char *a_syminput;
+extern enum symbol_order_t a_symbol_order;
 
 extern char *a_blockfile;
 
