@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2012 Tomaz Solc                                      */
+/* Copyright (C) 2007-2018 Tomaz Solc                                      */
 
 /* This program is free software; you can redistribute it and/or modify    */
 /* it under the terms of the GNU General Public License as published by    */
@@ -18,7 +18,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "symtab.h"
 #include "dz80.h"
 
 struct symbol *symbols=NULL;
@@ -56,17 +55,7 @@ void symbol_remove(struct symbol *symb)
 
 struct symbol *symbol_find(int val)
 {
-	struct symbol *cur;
-
-	cur=symbols;
-	while(cur!=NULL) {
-		if(cur->val == val) {
-			return cur;
-		}
-		cur=cur->next;
-	}
-
-	return NULL;
+	return symbol_find_next(val, NULL);
 }
 
 struct symbol *symbol_find_next(int val, struct symbol *cur)
@@ -87,6 +76,21 @@ struct symbol *symbol_find_next(int val, struct symbol *cur)
 	return NULL;
 }
 
+/* Find a symbol by name. Returns NULL if not found. */
+static struct symbol *symbol_find_name(const char *name)
+{
+	struct symbol *cur = symbols;
+
+	while(cur != NULL) {
+		if(!strcmp(cur->name, name)) {
+			return cur;
+		}
+		cur = cur->next;
+	}
+
+	return NULL;
+}
+
 int symbol_compare(struct symbol *a, struct symbol *b)
 {
 	if(a->val > b->val) {
@@ -156,23 +160,30 @@ struct symbol *symbol_find_range(int val)
 	return NULL;
 }
 
+/* We store symbols in a linked list. Two symbols can point to the same
+ * address, but two symbols cannot share the same name. */
+
 struct symbol *symbol_new(char *name, int val, int weight, int included)
 {
 	struct symbol *prev, *next;
 	struct symbol *dest;
 
+	/* This might be slow - we're traversing the whole linked list for each
+	 * new symbol. But computers are fast nowadays, and just how many
+	 * symbols can a 64k program have? */
+	dest = symbol_find_name(name);
+	if(dest != NULL) {
+		if(dest->val != val) {
+			msg(0, "Warning: not redefining symbol '%s' "
+				"(existing value 0x%04x, new value 0x%04x).\n",
+				name, dest->val, val);
+		}
+		return NULL;
+	}
+
 	msg(2, "Debug: defining new symbol '%s' with value 0x%04x\n", 
 							name, val);
 
-	dest=symbol_find(val);
-	if(dest!=NULL) {
-		if(!strcmp(name, dest->name)) return NULL;
-		/*
-		msg(0, "Warning: two symbols with same value defined "
-			"('%s' and '%s')\n", name, dest->name);
-	 	*/
-	}
-
 	dest=calloc(1, sizeof(*dest));
 	if(dest==NULL) return NULL;
 
@@ -281,10 +292,37 @@ int symbol_export_nonlabels(FILE *f)
 	return 0;
 }
 
-int symbol_export(FILE *f)
+/* Converts linked list of symbols to an array of pointers. Array should be
+ * freed by the caller. */
+static int symbol_to_array(struct symbol ***symbol_array, int *symbol_num)
+{
+	struct symbol *cur = symbols;
+	int n;
+
+	(*symbol_num) = 0;
+	while(cur != NULL) {
+		(*symbol_num)++;
+		cur = cur->next;
+	}
+
+	(*symbol_array) = calloc((*symbol_num), sizeof(**symbol_array));
+
+	cur = symbols;
+	n = 0;
+	while(cur != NULL) {
+		(*symbol_array)[n] = cur;
+		cur = cur->next;
+		n++;
+	}
+
+	return 0;
+}
+
+static int symbol_export_array(FILE *f, struct symbol **symbol_array, int symbol_num)
 {
 	struct symbol *cur;
 	struct reference *r;
+	int n;
 
 	fprintf(f, "; Symbol file\n\n");
 
@@ -292,10 +330,10 @@ int symbol_export(FILE *f)
 
 	fprintf(f, "\n");
 
-	cur=symbols;
-	while(cur!=NULL) {
+	for(n = 0; n < symbol_num; n++) {
+		cur = symbol_array[n];
+
 		if(cur->included) {
-			cur=cur->next;
 			continue;
 		}
 
@@ -342,9 +380,55 @@ int symbol_export(FILE *f)
 			r=r->next;
 		}
 		fprintf(f, "\n");
-
-		cur=cur->next;
 	}
 
-	return 1;
+	return 0;
+}
+
+static int symbol_get_ref_num(const struct symbol *s)
+{
+	const struct reference *r = s->ref;
+	int n = 0;
+
+	while(r != NULL) {
+		n++;
+		r = r->next;
+	}
+
+	return n;
+}
+
+static int cmp_usage_freq(const void *a, const void *b)
+{
+	const struct symbol *as = *((const struct symbol **) a);
+	const struct symbol *bs = *((const struct symbol **) b);
+
+	int an = symbol_get_ref_num(as);
+	int bn = symbol_get_ref_num(bs);
+
+	return bn - an;
+}
+
+int symbol_export(FILE *f, enum symbol_order_t order)
+{
+	struct symbol **symbol_array;
+	int symbol_num;
+	int rv;
+
+	symbol_to_array(&symbol_array, &symbol_num);
+
+	switch(order) {
+		case symbol_order_default:
+			break;
+		case symbol_order_frequency:
+			qsort(symbol_array, symbol_num, sizeof(*symbol_array),
+					cmp_usage_freq);
+			break;
+	}
+
+	rv = symbol_export_array(f, symbol_array, symbol_num);
+
+	free(symbol_array);
+
+	return rv;
 }
