@@ -26,6 +26,11 @@ enum referencetype {
 	cstdfw  /* used in a defw directive */
 };
 
+enum symbol_order_t {
+	symbol_order_default,
+	symbol_order_frequency
+};
+
 struct symbol {
 	char *name;
 	int automatic;
@@ -81,7 +86,7 @@ int symbol_setlabel(int val, int range);
 
 void symbol_remove_nonlabels();
 
-int symbol_export(FILE *f);
+int symbol_export(FILE *f, enum symbol_order_t order);
 int symbol_export_nonlabels(FILE *f);
 void symbol_export_includes(FILE *f);
 
